# Copyright (c) 2014 by Oliver Pfeiffer. All rights reserved.
#
# Permission to use, copy, modify, and distribute this software for any 
# purpose and without fee is hereby granted, provided that this copyright 
# notice appear in all copies.
#
# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED 
# WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF 
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
# ----------------------------------------------------------------------------
# Name    : Flatten Hierarchy
# Date    : 2014-05-21
# Version : 1.0.0
#
# Traverses through the selected entities and deeply explodes all nested
# groups and components that only have groups and components (but no actual
# geometry like edges or faces) as direct children. In other words this script
# flattens the hierarchy of the selected entities by bringing all nested
# groups and components up to the current level of selection without
# interfering any geometry.
#-----------------------------------------------------------------------------

require 'Sketchup'

module OPFH
	
	def self.is_container(entity)
		return entity.is_a?(Sketchup::Group) || 
			entity.is_a?(Sketchup::ComponentInstance)
	end

	def self.is_geometry(entity)
		return entity.is_a?(Sketchup::Edge) || 
			entity.is_a?(Sketchup::Face)
	end
	
	def self.get_children(entity)
		result = nil
		if (entity.is_a?(Sketchup::Group))
			result = entity.entities
		elsif (entity.is_a?(Sketchup::ComponentInstance))
			result = entity.definition.entities
		end
		return result
	end

	def self.is_qualified(entity)
		result = false
		children = get_children(entity)
		if (children)
			result = true
			children.each do |child|
				if (is_geometry(child))
					result = false
					break
				end
			end
		end
		return result
	end

	def self.explode_deeply(entity)
		if (is_qualified(entity))
			children = entity.explode
			selection = Sketchup.active_model.selection
			children.each do |child|
				if (is_container(child))
					explode_deeply(child)
					if (!child.deleted?)
						selection.add(child)
					end
				end
			end
		end
	end
	
	def self.flatten_hierarchy
		model = Sketchup.active_model
		selection = model.selection
		unless selection.empty?
			model.start_operation('Flatten Hierarchy')
			selection_copy = selection.to_a
			selection_copy.each do |entity|
				if (is_container(entity))
					explode_deeply(entity)
				end
			end
			model.commit_operation
		end
	end
	
end

unless file_loaded?((__FILE__))
	UI.add_context_menu_handler do |menu|
		model = Sketchup.active_model
		selection = model.selection
		unless selection.empty?
			found_container = false
			found_qualified = false
			selection.each do |entity|
				found_container |= !found_container && OPFH.is_container(entity)
				found_qualified |= !found_qualified && OPFH.is_qualified(entity)
			end
			if (found_container)
				item = menu.add_item('Flatten Hierarchy') { OPFH.flatten_hierarchy }
				menu.set_validation_proc(item)  {
					if (found_qualified)
						MF_ENABLED
					else
						MF_GRAYED
					end
				}
			end
		end
	end
	file_loaded((__FILE__))
end
